@@ -25,14 +25,17 @@ OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 THE SOFTWARE.
 """
 import re
+from typing import Any
+from typing import Dict
+from typing import Generator
+from typing import Optional
+from typing import Pattern
+from typing import Set
 
-from detect_secrets.core.potential_secret import PotentialSecret
-from detect_secrets.plugins.base import BasePlugin
-from detect_secrets.plugins.base import classproperty
-from detect_secrets.plugins.common.filetype import determine_file_type
-from detect_secrets.plugins.common.filetype import FileType
-from detect_secrets.plugins.common.filters import get_aho_corasick_helper
-from detect_secrets.plugins.common.filters import is_sequential_string
+from ..core.potential_secret import PotentialSecret
+from ..util.filetype import determine_file_type
+from ..util.filetype import FileType
+from .base import BasePlugin
 
 
 # Note: All values here should be lowercase
@@ -144,7 +147,7 @@ CLOSING = r'[]\'"]{0,2}'
 DENYLIST_REGEX = r'|'.join(DENYLIST)
 # Non-greedy match
 OPTIONAL_WHITESPACE = r'\s*?'
-OPTIONAL_NON_WHITESPACE = r'[^\s]*?'
+OPTIONAL_NON_WHITESPACE = r'[^\s]{0,50}?'
 QUOTE = r'[\'"]'
 SECRET = r'[^\s]+'
 SQUARE_BRACKETS = r'(\[\])'
@@ -256,33 +259,7 @@ class KeywordDetector(BasePlugin):
     """
     secret_type = 'Secret Keyword'
 
-    @classproperty
-    def default_options(cls):
-        return {
-            'keyword_exclude': None,
-        }
-
-    @property
-    def __dict__(self):
-        output = {
-            'keyword_exclude': self.keyword_exclude,
-        }
-        output.update(super(KeywordDetector, self).__dict__)
-
-        return output
-
-    def __init__(self, keyword_exclude=None, exclude_lines_regex=None, automaton=None, **kwargs):
-        false_positive_heuristics = [
-            get_aho_corasick_helper(automaton),
-            is_sequential_string,
-        ]
-
-        super(KeywordDetector, self).__init__(
-            exclude_lines_regex=exclude_lines_regex,
-            false_positive_heuristics=false_positive_heuristics,
-            **kwargs
-        )
-
+    def __init__(self, keyword_exclude: Optional[str] = None) -> None:
         self.keyword_exclude = None
         if keyword_exclude:
             self.keyword_exclude = re.compile(
@@ -290,33 +267,41 @@ class KeywordDetector(BasePlugin):
                 re.IGNORECASE,
             )
 
-        self.automaton = automaton
+    def analyze_string(
+        self,
+        string: str,
+        denylist_regex_to_group: Optional[Dict[Pattern, int]] = None,
+    ) -> Generator[str, None, None]:
+        if self.keyword_exclude and self.keyword_exclude.search(string):
+            return
 
-    def analyze_string_content(self, string, line_num, filename):
-        output = {}
-        if (
-            self.keyword_exclude
-            and self.keyword_exclude.search(string)
-        ):
-            return output
-        for identifier in self.secret_generator(
-            string,
-            filetype=determine_file_type(filename),
-        ):
-            if self.is_secret_false_positive(identifier):
-                continue
-            secret = PotentialSecret(
-                self.secret_type,
-                filename,
-                identifier,
-                line_num,
-            )
-            output[secret] = secret
+        if denylist_regex_to_group is None:
+            attempts = [
+                QUOTES_REQUIRED_DENYLIST_REGEX_TO_GROUP,
+                DENYLIST_REGEX_TO_GROUP,
+            ]
+        else:
+            attempts = [denylist_regex_to_group]
 
-        return output
+        has_results = False
+        for denylist_regex_to_group in attempts:
+            for denylist_regex, group_number in denylist_regex_to_group.items():
+                match = denylist_regex.search(string)
+                if match:
+                    has_results = True
+                    yield match.group(group_number)
 
-    def secret_generator(self, string, filetype):
-        lowered_string = string.lower()
+            if has_results:
+                break
+
+    def analyze_line(
+        self,
+        filename: str,
+        line: str,
+        line_number: int = 0,
+        **kwargs: Any,
+    ) -> Set[PotentialSecret]:
+        filetype = determine_file_type(filename)
 
         if filetype in QUOTES_REQUIRED_FILETYPES:
             denylist_regex_to_group = QUOTES_REQUIRED_DENYLIST_REGEX_TO_GROUP
@@ -327,71 +312,19 @@ class KeywordDetector(BasePlugin):
         else:
             denylist_regex_to_group = DENYLIST_REGEX_TO_GROUP
 
-        for denylist_regex, group_number in denylist_regex_to_group.items():
-            match = denylist_regex.search(lowered_string)
-            if match:
-                lowered_secret = match.group(group_number)
-
-                # ([^\s]+) guarantees lowered_secret is not ''
-                if not probably_false_positive(
-                    lowered_secret,
-                    filetype=filetype,
-                ):
-                    yield lowered_secret
-
-
-def probably_false_positive(lowered_secret, filetype):
-    if (
-        any(
-            false_positive in lowered_secret
-            for false_positive in (
-                '/etc/',
-                'fake',
-                'forgot',
-            )
-        ) or lowered_secret in FALSE_POSITIVES
-        # For e.g. private_key "some/dir/that/is/not/a/secret";
-        or lowered_secret.count('/') >= 3
-        # For e.g. "secret": "{secret}"
-        or (
-            lowered_secret[0] == '{'
-            and lowered_secret[-1] == '}'
-        ) or (
-            filetype not in QUOTES_REQUIRED_FILETYPES
-            and lowered_secret[0] == '$'
-        ) or (
-            filetype == FileType.EXAMPLE
-            and lowered_secret[0] == '<'
-            and lowered_secret[-1] == '>'
+        return super().analyze_line(
+            filename=filename,
+            line=line,
+            line_number=line_number,
+            denylist_regex_to_group=denylist_regex_to_group,
         )
-    ):
-        return True
 
-    # Heuristic for no function calls
-    try:
-        if (
-            lowered_secret.index('(') < lowered_secret.index(')')
-        ):
-            return True
-    except ValueError:
-        pass
-
-    # Heuristic for e.g. request.json_body['hey']
-    try:
-        if (
-            lowered_secret.index('[') < lowered_secret.index(']')
-        ):
-            return True
-    except ValueError:
-        pass
-
-    # Heuristic for e.g. ${link}
-    try:
-        if (
-            lowered_secret.index('${') < lowered_secret.index('}')
-        ):
-            return True
-    except ValueError:
-        pass
-
-    return False
+    def json(self) -> Dict[str, Any]:
+        return {
+            'keyword_exclude': (
+                self.keyword_exclude.pattern
+                if self.keyword_exclude
+                else ''
+            ),
+            **super().json(),
+        }
